#!/usr/bin/env python3
#
# Class to simplify RESTful API communication with all Montana Instruments instruments.
#
# Example usage:
#  import instrument
#  inst = instrument.Instrument('192.168.45.123', port='47101', version='v1')
#  inst.call_method('/controller/methods/cooldown()')
#
import os
import os.path as op
import sys

from enum import IntEnum
import requests
import json
import re
import time

from . import ssh_tunnel
import socket
import http

from . import mirs_helpers

_force_ipv4 = True  # Some day IPv6 will be ubiquitous.  Today is not that day.

_success = range(200,300)


class Rest_Ports(IntEnum):
    """Constants for TCP port numbers of the various REST servers
    """
    scryostation_hlm       = 47101  # /etc/services entry is "cryostation-hlm      47101/tcp"
    cryocore_hlm           = 47101  # /etc/services entry is "cryostation-hlm      47101/tcp"
    status_leds_llm        = 47102  # /etc/services entry is "statusleds-llm       47102/tcp"
    xpcryostation_hlm      = 47103  # /etc/services entry is "sub2k-hlm            47103/tcp"
    positioner_hlm         = 47104  # /etc/services entry is "positioner-hlm       47104/tcp"
    power_hub_llm          = 47105  # /etc/services entry is "powerhub-llm         47105/tcp"
    tcm_cal_llm            = 47107  # /etc/services entry is "tcmcal-llm           47107/tcp"

    temp_control1_llm      = 47111  # /etc/services entry is "tcm1-llm             47111/tcp"
    temp_control2_llm      = 47112  # /etc/services entry is "tcm2-llm             47112/tcp"
    temp_control3_llm      = 47113  # /etc/services entry is "tcm3-llm             47113/tcp"
    temp_control4_llm      = 47114  # /etc/services entry is "tcm4-llm             47114/tcp"
    temp_control5_llm      = 47115  # /etc/services entry is "tcm5-llm             47115/tcp"
    
    piezo_stepper1_llm     = 47121 # /etc/services entry is "piezostep1-llm        47121/tcp"
    piezo_stepper2_llm     = 47122 # /etc/services entry is "piezostep2-llm        47122/tcp"
    piezo_stepper3_llm     = 47123 # /etc/services entry is "piezostep3-llm        47123/tcp"
    piezo_stepper4_llm     = 47124 # /etc/services entry is "piezostep4-llm        47124/tcp"
    piezo_stepper5_llm     = 47125 # /etc/services entry is "piezostep5-llm        47125/tcp"
    
    pfeiffertc110_llm      = 47130 # /etc/services entry is "pfeiffertc110-llm     47130/tcp"
    
    cryostatancillary_llm  = 47135 # /etc/services entry is "cryostatancillary-llm 47135/tcp"
    vacuumcontrol_llm      = 47136 # /etc/services entry is "vacuumcontrol-llm     47136/tcp"
    thermalswitch_llm      = 47137 # /etc/services entry is "thermalswitch-llm     47137/tcp"
    
    massflow_llm           = 47140 # /etc/services entry is "massflow-llm          47140/tcp"

    mercury_hlm            = 47150 # /etc/services entry is "mercury-hlm           47150/tcp"

    keyencecl3000_llm      = 47160 # /etc/services entry is "keyencecl3000-llm     47160/tcp"
    keyencesift1000_llm    = 47161 # /etc/services entry is "keyencesift1000-llm   47161/tcp"
    
    mmc113_llm             = 47170 # /etc/services entry is "mmc113-llm            47170/tcp"
    lynx_hlm               = 47171 # /etc/services entry is "lynx-hlm              47171/tcp"
    
    chronos_hlm            = 47180  # /etc/services entry is "chronos-hlm          47180/tcp"

    system_hlm             = 47210  # /etc/services entry is "system-hlm           47210/tcp"

    pegasus_hlm            = 47220  #  /etc/services entry is "pegasus-hlm         47220/tcp"

    pass


TunnelError = ssh_tunnel.TunnelError

class CouldNotConnect(Exception): pass

class NotConnected(Exception): pass

class BadUrl(Exception): pass

def _verifySuccess(op, path, resp) -> None:
    """Make sure the operation succeeded, otherwise throw ApiError exception

    Keyword arguments:
       op   -- REST operation that occurred
       path -- The URI of the operation
       resp -- Response from the server (body and HTTP response code)
    """
    if resp.status_code not in _success:
        # Something went wrong

        brief = None
        details = None
        try:
            if 'title' in resp.json().keys(): brief = resp.json()['title']
            if 'detail' in resp.json().keys(): details = resp.json()['detail']
        except:
            pass

        raise ApiError(op, path, resp.status_code, brief, details)

    return

def _rewrite_connection(ip, port, tunnel):
    """For hostnames that start with "mirs:" determine port on MIRS server

    If the user specified "mirs:somemachine" this routine will lookup 
    the corresponding secure tunnel on the MIRS server.

    If ip doesn't start with "mirs:" then ip, port, and tunnel are
    returned unchanged. 
    """
    
    if isinstance(ip, str):
        if ip.startswith("mirs:"):
            (miec, mirs) = (ip+":support").split(':')[1:3]
            try:
                (ip, port, tunnel) = (mirs, 
                                      mirs_helpers.lookup_port(miec, port, mirs_server=mirs),
                                      False)
            except mirs_helpers.MirsClientNotFound:
                pass
        elif _force_ipv4 and not re.match('[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$', ip):
            ip = socket.getaddrinfo(ip, None, socket.AF_INET)[0][4][0]
    return (ip, port, tunnel)
        
class Instrument:
    def __init__(self, ip, port, version, verbose=False, tunnel=False):
        """Create a base instrument object for Scripting R3

        Keyword arguments:
          ip      -- IP address (or hostname) of the instrument (e.g. an XP100).
                     To access the instrumnent through a remote access server
                     use a string like 
                        mirs:<miec>:<name or IP of server>
                     e.g. 
                        mirs:CR-713:support
                     If the name of the MIRS is omitted it defaults 
                     to support
          port    -- TCP Port number of the service on the MIEC.                     
          version -- REST API version (e.g. "v1")
          verbose -- Print debug information?
          tunnel  -- Communicate through a secure SSH tunnel?
        """
        assert not isinstance(port, (list,tuple)), "This class can only talk to a single port"

        self.tun     = None
        (self.ip, self.port, tunnel) = _rewrite_connection(ip, port, tunnel)

        self.version = version
        self.verbose = verbose

        if tunnel:
            try:
                self.tun  = ssh_tunnel.tunnel(remote_host = ip, remote_ports = port)
            except:
                raise TunnelError("Could not start the ssh tunnel.  Have you copied your ssh keys?")
            self.ip   = self.tun.local_bind_address[0]
            self.port = self.tun.local_bind_ports[0]
        return

    def close(self):
        if self.tun is not None:
            self.tun.stop()
        self.ip      = None
        self.port    = None
        self.version = None
        self.verbose = None
        self.tun     = None


    def __del__(self):
        self.close()
        return

    def __root_endpoint(self):
        """The portion common to all endpoints for this instrument.

        Example:
          self.__root_endpoint() -> "http://192.168.45.244:47101/v1"
        """
        return f'http://{self.ip}:{self.port}/{self.version}'

    def is_up(self):
        """Is the instrument responding?
        """
        result = False

        try:
            tmp = self.get("/version")
            result = True
        except ApiError:
            # 
            # An API error means the cryostation didn't like what we
            # sent it.  The fact that it doesn't like something means
            # it is up and running.
            # 
            result = True
        except http.client.RemoteDisconnected:
            self.close()
            pass
        except requests.exceptions.ConnectionError:
            self.close()
            pass
        except NotConnected:
            print("hit an instrument.NotConnected")
            self.close()
            pass            
        except:
            raise
        return result

    def url(self, path):
        if self.ip is None: raise NotConnected()

        root = self.__root_endpoint()

        path = path.strip('/')
        path = path.replace('//', '/')

        return f'{root}/{path}'

    def call_method(self, path, data=None):
        """Call a method to perform some action with the instrument.

        """
        uri = self.url(path)
        if (data is not None):
            if (self.verbose): print(f'POST {uri} [data -> {data}]')
            resp = requests.post(uri, json=data)
        else:
            if (self.verbose): print(f'POST {uri}')
            resp = requests.post(uri)

        _verifySuccess('POST', path, resp)

        c = resp.content.decode('utf-8')
        if c != '':
            return json.loads(c)

    def set_prop(self, path, data=None):
        uri = self.url(path)

        if (self.verbose): print(f'PUT {uri} [data -> {data}]')
        resp = requests.put(uri, json=data)

        _verifySuccess('PUT', path, resp)

        c = resp.content.decode('utf-8')
        if c != '':
            return json.loads(c)

    def get_prop(self, path, params=None):
        """Retrieve a property value via REST from the instrument

        Keyword arguments:
           path    The non-root portion of the URI.
                   e.g. "/controller1/thermometer/properties/sample"
           params  (optional) Any parameters that need to be passed
                   along in the GET message.
        """
        uri = self.url(path)
        if (self.verbose): print(f'GET {uri} [params -> {params}]')
        try:
            # print(f"uri is |{uri}|")
            resp = requests.get(uri, params)
        except requests.exceptions.InvalidURL:
            raise BadUrl() from None
        except requests.exceptions.ConnectionError:
            raise
        except requests.exceptions.InvalidURL:
            raise BadUrl()
        except:
            sys.stderr.write("unknown exception\n")
            sys.stderr.write(f"{sys.exc_info()}")
            raise

        _verifySuccess('GET', path, resp)

        c = resp.content.decode('utf-8')
        result = None
        if c != '': result = json.loads(c)

        return result

    def get(self, path, params=None):
        """Deprecated.  Use get_prop() instead."""
        return self.get_prop(path, params)


class ApiError(Exception):
    """An API Error Exception"""

    def __init__(self, op, path, response_code, brief, details):
        """Create an error object.

        Note- Normally this is not directly called.  The wrapper
        _verifySuccess will, if needed, create and raise this
        type of exception.

        Keyword arguments:
           op            -- REST operation that triggered the error (e.g. "GET").
           path          -- The URI that triggered the error.
           response_code -- The HTTP response code (e.g. 400)
           brief            A synopsis of the problem.  Often it is the
                            contents of the "title" field of message content.
           details       -- A description of the problem.  Often it is the
                            contents of the "detail" field of message content.
        """
        self.op            = op
        self.path          = path
        self.response_code = response_code
        self.brief         = brief
        self.details       = details
        return

    def __str__(self):
        result = f"ApiError: {self.op}: {self.response_code}"
        if self.brief: result = f"ApiError: {self.op}: {self.response_code}: {self.brief}"
        return result



from qtpy import QtWidgets

from pymodaq_gui import utils as gutils
from pymodaq_utils.config import Config, ConfigError
from pymodaq_utils.logger import set_logger, get_module_name

from pymodaq.utils.config import get_set_preset_path
from pymodaq.extensions.utils import CustomExt


# todo: replace here *pymodaq_plugins_template* by your plugin package name
from pymodaq_plugins_template.utils import Config as PluginConfig

logger = set_logger(get_module_name(__file__))

main_config = Config()
plugin_config = PluginConfig()

# todo: modify this as you wish
EXTENSION_NAME = 'MY_EXTENSION_NAME'  # the name that will be displayed in the extension list in the
# dashboard
CLASS_NAME = 'CustomExtensionTemplate'  # this should be the name of your class defined below


# todo: modify the name of this class to reflect its application and change the name in the main
# method at the end of the script
class CustomExtensionTemplate(CustomExt):

    # todo: if you wish to create custom Parameter and corresponding widgets. These will be
    # automatically added as children of self.settings. Morevover, the self.settings_tree will
    # render the widgets in a Qtree. If you wish to see it in your app, add is into a Dock
    params = []

    def __init__(self, parent: gutils.DockArea, dashboard):
        super().__init__(parent, dashboard)

        # info: in an extension, if you want to interact with ControlModules you have to use the
        # object: self.modules_manager which is a ModulesManager instance from the dashboard

        self.setup_ui()

    def setup_docks(self):
        """Mandatory method to be subclassed to setup the docks layout

        Examples
        --------
        >>>self.docks['ADock'] = gutils.Dock('ADock name')
        >>>self.dockarea.addDock(self.docks['ADock'])
        >>>self.docks['AnotherDock'] = gutils.Dock('AnotherDock name')
        >>>self.dockarea.addDock(self.docks['AnotherDock'''], 'bottom', self.docks['ADock'])

        See Also
        --------
        pyqtgraph.dockarea.Dock
        """
        # todo: create docks and add them here to hold your widgets
        # reminder, the attribute self.settings_tree will  render the widgets in a Qtree.
        # If you wish to see it in your app, add is into a Dock
        raise NotImplementedError

    def setup_actions(self):
        """Method where to create actions to be subclassed. Mandatory

        Examples
        --------
        >>> self.add_action('quit', 'Quit', 'close2', "Quit program")
        >>> self.add_action('grab', 'Grab', 'camera', "Grab from camera", checkable=True)
        >>> self.add_action('load', 'Load', 'Open', "Load target file (.h5, .png, .jpg) or data from camera"
            , checkable=False)
        >>> self.add_action('save', 'Save', 'SaveAs', "Save current data", checkable=False)

        See Also
        --------
        ActionManager.add_action
        """
        raise NotImplementedError(f'You have to define actions here')

    def connect_things(self):
        """Connect actions and/or other widgets signal to methods"""
        raise NotImplementedError

    def setup_menu(self, menubar: QtWidgets.QMenuBar = None):
        """Non mandatory method to be subclassed in order to create a menubar

        create menu for actions contained into the self._actions, for instance:

        Examples
        --------
        >>>file_menu = menubar.addMenu('File')
        >>>self.affect_to('load', file_menu)
        >>>self.affect_to('save', file_menu)

        >>>file_menu.addSeparator()
        >>>self.affect_to('quit', file_menu)

        See Also
        --------
        pymodaq.utils.managers.action_manager.ActionManager
        """
        # todo create and populate menu using actions defined above in self.setup_actions
        pass

    def value_changed(self, param):
        """ Actions to perform when one of the param's value in self.settings is changed from the
        user interface

        For instance:
        if param.name() == 'do_something':
            if param.value():
                print('Do something')
                self.settings.child('main_settings', 'something_done').setValue(False)

        Parameters
        ----------
        param: (Parameter) the parameter whose value just changed
        """
        pass


def main():
    from pymodaq.utils.gui_utils.utils import mkQApp
    from pymodaq.utils.gui_utils.loader_utils import load_dashboard_with_preset
    from pymodaq.utils.messenger import messagebox

    app = mkQApp(EXTENSION_NAME)
    try:
        preset_file_name = plugin_config('presets', f'preset_for_{CLASS_NAME.lower()}')
        load_dashboard_with_preset(preset_file_name, EXTENSION_NAME)
        app.exec()

    except ConfigError as e:
        messagebox(f'No entry with name f"preset_for_{CLASS_NAME.lower()}" has been configured'
                   f'in the plugin config file. The toml entry should be:\n'
                   f'[presets]'
                   f"preset_for_{CLASS_NAME.lower()} = {'a name for an existing preset'}"
                   )


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

"""Routines to perform real-time plotting of  data from a temperature controller"""

import matplotlib.pyplot as plt
import matplotlib.animation as animation
from matplotlib.ticker import FormatStrFormatter
import time
import datetime as dt

_line_styles = ['r-',
                'b-', 
                'g-', 
                'c-', 
                'm-', 
                'y-', 
                'k-']

class TcPlot():
    def __init__(self, heater_names, thermometer_names, duration=120):
        """
        
        Keyword Parameters: 
           heater_names
           thermometer_names
           duration            (in seconds)
        """
        assert (len(heater_names)+len(thermometer_names)) <= len(_line_styles), f"We don't have enough colors for {len(samples)} samples.  Maximum is {len(_line_styles)}."

        self.heater_names      = heater_names
        self.thermometer_names = thermometer_names
        self.x                 = []
        self.ys                = [[]] * (len(heater_names) + len(thermometer_names))

        self.Span = duration      # timespan to plot (in seconds)
        self.PaintInterval = 1.0  # period between plot updates (in seconds)
        self.Closed = False       # has user closed the plot window?
        self.OnClosed = None      # callback function called on window close.

        # Create the plot
        self.fig, self.ax1 = plt.subplots()

        # Hook the close click event
        self.fig.canvas.mpl_connect('close_event', self.handle_close)

        # Enable interactive mode
        plt.ion()

        # Init primary axis
        self.ax1.set_xlabel('time (s)')                
        self.ax1.set_ylabel('Heater (W)', color='r')
        self.ax1.yaxis.set_major_formatter(FormatStrFormatter('%.4f'))
        self.ax1.tick_params('y', colors='r')
        self.ax1.grid(color='r', linestyle=':', linewidth=1)
        
        # Init secondary axis
        self.ax2 = self.ax1.twinx()
        self.ax2.set_ylabel('Temp (K)', color='b')
        self.ax2.yaxis.set_major_formatter(FormatStrFormatter('%.4f'))
        self.ax2.tick_params('y', colors='b')
        self.ax2.grid(color='b', linestyle=':', linewidth=1)
        
        # Create a line for each heater and sensor
        self.lines = [None] * len(self.ys)
        for i in range(len(self.lines)):
            self.lines[i], = self._axis(i).plot(self.x, self.ys[i], _line_styles[i])

        # Add a legend to the plot.
        labels = [f"{h} (W)" for h in self.heater_names]+[f"{t} (K)" for t in self.thermometer_names]
        plt.legend(self.lines, labels, loc='upper left')

        self.clear()
        plt.show()
        
        self.lastPaintTime = dt.datetime.now()

    def _axis(self, i):
        """Lookup which axis to plot the i'th line onto.

        Heaters plot on the first axis, thermometers plot on the second.

        Keyword Parameters:
          i   index into the lines array.
        """
        ax = self.ax1
        if i >= len(self.heater_names): ax = self.ax2
        return ax

        
    def setWindowTitle(self, title):
        self.fig.canvas.setWindowTitle(title)

    def handle_close(self, evt):
        # Indicate the plot has been closed
        self.Closed = True

        # Call event handler, if set
        if self.OnClosed is not None:
            self.OnClosed()

    def clear(self):
        
        # Remove all existing data
        self.x  = []
        self.ys = [[]] * len(self.ys)
        
        # Capture the start time to calculate elapsed time later within update()
        self.start = dt.datetime.now()

    def update(self, heaters, thermometers):
        """y1 = Heater power, y2 = Temperature"""

        samples = heaters + thermometers

        assert len(samples) == len(self.ys), f"Must provide a total of {len(samples)} samples."
        
        if self.Closed:
            return

        # Capture the total elapsed time since the plot has been started
        n      = dt.datetime.now()
        deltaT = (n - self.start)
        elapsed = deltaT.total_seconds()

        # Capture the minimum timestamp of data allowed to remain within the plot
        minsec = elapsed - self.Span

        # Delete all data that is too old
        while len(self.x) > 0 and self.x[0] < minsec:
            self.x = self.x[1:]
            for i in range(len(self.ys)):
                self.ys[i] = self.ys[i][1:]

        # Add the new samples
        self.x.append(elapsed)
        for i in range(len(self.ys)):
            self.ys[i] = self.ys[i] + [samples[i]]

        # Calc the time elasped since the last paint of the plot
        elapsed = (dt.datetime.now() - self.lastPaintTime).total_seconds()

        if elapsed >= self.PaintInterval:
            # Repaint the plot

            self.lastPaintTime = dt.datetime.now()

            for i in range(len(self.lines)):
                self.lines[i].set_xdata(self.x)
                self.lines[i].set_ydata(self.ys[i])

            self.ax1.relim()
            self.ax1.autoscale_view()

            self.ax2.relim()
            self.ax2.autoscale_view()

        # Redraw        
        self.fig.canvas.draw()
        plt.pause(0.001)




#!/usr/bin/env python3


import os
import sys
import __main__

import PySide6.QtCore     as qtc
import PySide6.QtUiTools  as qtut


def resource_path(relative_path):
    """Return path of a resource file

    This is needed for programs "compiled" with pyinstaller with the
    --onefile option.  When running the program normally the required
    files (e.g. mydialog.ui) are located in a location relative to the
    the script.  But when running an .exe generated by pyinstaller the
    required files are unpacked into a temporary directory.

    This function determines if the script is being run normally or if
    is is a pyinstaller generated exectuble and will return the
    correct path for the requried files.

    """
    default_dir = os.path.dirname(os.path.abspath(__main__.__file__))
    mei_dir     = getattr(sys, '_MEIPASS', default_dir)
    return os.path.join(mei_dir, relative_path)


def load_ui(ui_fname, custom_widgets = None):
    """Create a Qt Window from a .ui file.

    Keyword Parameters
      ui_fname       -- Name of the user interface (i.e. the .ui) file.
      custom_widgets -- List of custom widgets (i.e. wdigets not 
                        natively known to QUILoader) to register with
                        the loader.  (e.g. QChartView) 
    """
    if custom_widgets is None: custom_widgets = []

    f = qtc.QFile(ui_fname)
    loader = qtut.QUiLoader()
    for w in custom_widgets:
        loader.registerCustomWidget(w)
    window = loader.load(f)
    f.close()
    return window
    



import numpy as np

from pymodaq.extensions.pid.utils import PIDModelGeneric, DataToActuatorPID, main
from pymodaq_data.data import DataToExport, DataCalculated

from pymodaq.utils.data import DataActuator

from typing import List


def some_function_to_convert_the_pid_outputs(outputs: List[float], dt: float, stab=True):
    """ Should be replaced here or in the model class to process the outputs """
    return outputs


def some_function_to_convert_the_data(measurements: DataToExport):
    """ Should be replaced here or in the model class to process the measurement """
    a = 0
    b = 1
    return [a, b]


class PIDModelTemplate(PIDModelGeneric):
    limits = dict(max=dict(state=False, value=100),
                  min=dict(state=False, value=-100),)
    konstants = dict(kp=0.1, ki=0.000, kd=0.0000)

    Nsetpoints = 2  # number of setpoints
    setpoint_ini = [128, 128]  # number and values of initial setpoints
    setpoints_names = ['Xaxis', 'Yaxis']  # number and names of setpoints

    actuators_name = ["Xpiezo", "Ypiezo"]  # names of actuator's control modules involved in the PID
    detectors_name = ['Camera']  # names of detector's control modules involved in the PID

    params = []  # list of dict to initialize specific Parameters

    def __init__(self, pid_controller):
        super().__init__(pid_controller)

    def update_settings(self, param):
        """
        Get a parameter instance whose value has been modified by a user on the UI
        Parameters
        ----------
        param: (Parameter) instance of Parameter object
        """
        if param.name() == '':
            pass

    def ini_model(self):
        super().ini_model()

        # add here other specifics initialization if needed

    def convert_input(self, measurements: DataToExport):
        """
        Convert the measurements in the units to be fed to the PID (same dimensionality as the setpoint)
        Parameters
        ----------
        measurements: DataToExport
            Data from the declared detectors from which the model extract a value of the same units as the setpoint

        Returns
        -------
        InputFromDetector: the converted input in the setpoints units

        """

        x, y = some_function_to_convert_the_data(measurements)
        return DataToExport('pid inputs',
                            data=[DataCalculated('pid calculated',
                                                 data=[np.array([x]),
                                                       np.array([y])])])

    def convert_output(self, outputs: List[float], dt: float, stab=True) -> DataToActuatorPID:
        """
        Convert the output of the PID in units to be fed into the actuator
        Parameters
        ----------
        outputs: List of float
            output value from the PID from which the model extract a value of the same units as the actuator
        dt: float
            Ellapsed time since the last call to this function
        stab: bool

        Returns
        -------
        OutputToActuator: the converted output

        """
        outputs = some_function_to_convert_the_pid_outputs(outputs, dt, stab)
        return DataToActuatorPID('pid output', mode='rel',
                                 data=[DataActuator(self.actuators_name[ind], data=outputs[ind])
                                       for ind in range(len(outputs))])


if __name__ == '__main__':
    main("BeamSteeringMockNoModel.xml")  # some preset configured with the right actuators and detectors





#!/usr/bin/env python3
#

import sys
import os
import instrument
import time
import datetime

class Pegasus(instrument.Instrument):
    def __init__(self, ip, version='v1', verbose=False, tunnel=False):
        super().__init__(ip=ip,
                         port=instrument.Rest_Ports.pegasus_hlm,
                         version=version,
                         verbose=verbose,
                         tunnel=tunnel)

    def start_cryocooler(self):
        '''Start running the cryocooler'''
        self.call_method('cooler/cryocooler/methods/startCryocooler()')
        return

    def stop_cryocooler(self):
        '''Stop running the cryocooler'''
        self.call_method('cooler/cryocooler/methods/stopCryocooler()')
        return

    def get_compressor_speed(self):
        """Get the current compressor speed (Hz)"""
        r = self.get_prop('/cooler/cryocooler/properties/compressorSpeed')
        return r['compressorSpeed']

    def get_cryocooler_running(self):
        """Get the cryocooler running status"""
        r = self.get_prop('/cooler/cryocooler/properties/cryocoolerRunning')
        return r['cryocoolerRunning']

    def get_device_connected(self):
        """Get the device connected status"""
        r = self.get_prop('/cooler/cryocooler/properties/deviceConnected')
        return r['deviceConnected']

    def get_max_allowed_compressor_speed(self):
        """Get the maximum allowed compressor speed (Hz)"""
        r = self.get_prop('/cooler/cryocooler/properties/maxAllowedCompressorSpeed')
        return r['maxAllowedCompressorSpeed']

    def get_min_allowed_compressor_speed(self):
        """Get the minimum allowed compressor speed (Hz)"""
        r = self.get_prop('/cooler/cryocooler/properties/minAllowedCompressorSpeed')
        return r['minAllowedCompressorSpeed']

    def get_power_consumption(self):
        """Get the power consumption (W)"""
        r = self.get_prop('/cooler/cryocooler/properties/powerConsumption')
        return r['powerConsumption']

    def get_return_pressure(self):
        """Get the return pressure (Pa)"""
        r = self.get_prop('/cooler/cryocooler/properties/returnPressure')
        return r['returnPressure']

    def get_supply_pressure(self):
        """Get the supply pressure (Pa)"""
        r = self.get_prop('/cooler/cryocooler/properties/supplyPressure')
        return r['supplyPressure']

    def get_target_compressor_speed(self):
        """Get the target compressor speed (Hz)"""
        r = self.get_prop('/cooler/cryocooler/properties/targetCompressorSpeed')
        return r['targetCompressorSpeed']

    def set_target_compressor_speed(self, target):
        """Set the target compressor speed (Hz)"""
        self.set_prop('/cooler/cryocooler/properties/targetCompressorSpeed', target)
        return

    def get_water_inlet_temperature(self):
        """Get the water inlet temperature (K)"""
        r = self.get_prop('/cooler/cryocooler/properties/waterInletTemperature')
        return r['waterInletTemperature']

    def get_water_outlet_temperature(self):
        """Get the water outlet temperature (K)"""
        r = self.get_prop('/cooler/cryocooler/properties/waterOutletTemperature')
        return r['waterOutletTemperature']

    def print_status(self):
        """Print the current status of the cryocooler"""
        try:
            compressor_speed = self.get_compressor_speed()
            cryocooler_running = self.get_cryocooler_running()
            max_speed = self.get_max_allowed_compressor_speed()
            min_speed = self.get_min_allowed_compressor_speed()
            power_consumption = self.get_power_consumption()
            return_pressure = _pa_to_psig(self.get_return_pressure())
            supply_pressure = _pa_to_psig(self.get_supply_pressure())
            target_speed = self.get_target_compressor_speed()
            water_inlet_temp = _kelvin_to_celsius(self.get_water_inlet_temperature())
            water_outlet_temp = _kelvin_to_celsius(self.get_water_outlet_temperature())

            print(f"\n--- Status Update ({datetime.datetime.now()}) ---")
            print(f"Compressor Running: {cryocooler_running}")
            print(f"Compressor Speed: {compressor_speed} Hz")
            print(f"Target Compressor Speed: {target_speed} Hz")
            print(f"Supply Pressure: {supply_pressure:.2f} psig")
            print(f"Return Pressure: {return_pressure:.2f} psig")
            print(f"Differential Pressure: {return_pressure - supply_pressure:.2f} psig")
            print(f"Water Inlet Temperature: {water_inlet_temp:.2f} °C")
            print(f"Water Outlet Temperature: {water_outlet_temp:.2f} °C")
            print(f"Power Consumption: {power_consumption} W")
            print(f"Max Allowed Compressor Speed: {max_speed} Hz")
            print(f"Min Allowed Compressor Speed: {min_speed} Hz")
        except Exception as e:
            print(f"Error retrieving status: {e}")

            
def _pa_to_psig(pa):
    """Convert Pascals to pounds per square inch gauge (psig)"""
    return pa * 0.0001450377 


def _kelvin_to_celsius(kelvin):
    """Convert Kelvin to Celsius"""
    return kelvin - 273.15


if __name__ == "__main__":
    # Example to demonstrate basic usage:
    #  1. set target compressor speed to 78 Hz
    #  2. start the cryocooler
    #  3. loop for 15 minutes, printing status every 10 seconds
    #  4. stop the cryocooler
    try:
        pegasus = Pegasus('192.168.1.50')  # Replace with your system's IP address
        print("Pegasus instrument initialized")

        # Set target compressor speed to 78 Hz
        pegasus.set_target_compressor_speed(78)
        print("Target compressor speed set to 78 Hz")

        # Start the cryocooler
        pegasus.start_cryocooler()
        print("Cryocooler started")

        # Loop every 30 seconds for 15 minutes (30 * 30 = 900 seconds)
        start_time = time.time()
        while time.time() - start_time < 900:
            pegasus.print_status()
            time.sleep(10)

        # Stop the cryocooler
        pegasus.stop_cryocooler()
        print("Cryocooler stopped")

    except Exception as e:
        print(f"An error occurred: {e}")



#!/usr/bin/env python3
"""
Class to simplify RESTful API communication with the xp-series Cryostation instrument.

Example usage:
 from xpcryostation import *
 cryo = XPCryostation('192.168.1.123')

 # Shows how to use high-level methods for accessing common functions
 cryo.cooldown_sample_chamber()
 cryo.set_platform_target_temperature(1.7)
 cryo.get_platform_temperature()
 cryo.get_platform_temperature_stability()
 cryo.warmup_sample_chamber()

 # Shows how to use generic post/get/put methods for accessing any REST url/end-point
 cryo.call_method('/controller/methods/sampleChamberCooldown()')
 cryo.get_prop('/sampleChamber/temperatureControllers/platform/thermometer/sample')['sample']['temperatureAvg1Sec']
 cryo.set_prop('/controller/properties/platformTargetTemperature', 1.7)
"""

import sys
import os
import instrument
import genericcryostat

Ports = instrument.Rest_Ports

@genericcryostat.register
class XPCryostation(genericcryostat.GenericCryostat):
    def __init__(self, ip, version='v1', verbose=False, tunnel=False, port=Ports.xpcryostation_hlm):
        super().__init__(ip=ip,
                         port=port,
                         version=version,
                         verbose=verbose,
                         tunnel=tunnel)

    def get_sample_chamber_goal(self):
        r = self.get_prop('/controller/properties/sampleChamberGoal')
        return r['sampleChamberGoal']

    def get_sample_chamber_state(self):
        r = self.get_prop('/controller/properties/sampleChamberState')
        return r['sampleChamberState']

    def cooldown_sample_chamber(self):
        self.call_method('/controller/methods/cooldownSampleChamber()')

    def warmup_sample_chamber(self):
        self.call_method('/controller/methods/warmupSampleChamber()')

    def vent_sample_chamber(self):
        self.call_method('/controller/methods/ventSampleChamber()')

    def pull_vacuum_sample_chamber(self):
        self.call_method('/controller/methods/pullVacuumSampleChamber()')

    def abort_sample_chamber_goal(self):
        self.call_method('/controller/methods/abortSampleChamberGoal()')

    def get_sample_chamber_pressure(self):
        r = self.get_prop('/vacuumSystem/vacuumGauges/sampleChamberPressure/properties/pressureSample')
        return r['pressureSample']['pressure']

    def set_platform_bakeout_enabled(self, enabled):
        return self.set_prop('/controller/properties/platformBakeoutEnabled', enabled)

    def set_platform_bakeout_temperature(self, temperature):
        return self.set_prop('/controller/properties/platformBakeoutTemperature', temperature)

    def set_platform_bakeout_time(self, duration):
        return self.set_prop('/controller/properties/platformBakeoutTime', duration)

    def set_sample_chamber_dry_nitrogen_purge_enabled(self, enabled):
        return self.set_prop('/controller/properties/sampleChamberDryNitrogenPurgeEnabled', enabled)

    def set_sample_chamber_dry_nitrogen_purge_num_times(self, times):
        return self.set_prop('/controller/properties/sampleChamberDryNitrogenPurgeNumTimes', times)


    def get_cooler_goal(self):
        r = self.get_prop('/controller/properties/coolerGoal')
        return r['coolerGoal']

    def get_cooler_state(self):
        r = self.get_prop('/controller/properties/coolerState')
        return r['coolerState']

    def cooldown_cooler(self):
        self.call_method('/controller/methods/cooldownCooler()')

    def warmup_cooler(self):
        self.call_method('/controller/methods/warmupCooler()')

    def abort_cooler_goal(self):
        self.call_method('/controller/methods/abortCoolerGoal()')

    def get_cooler_pressure(self):
        r = self.get_prop('/vacuumSystem/vacuumGauges/coolerPressure/properties/pressureSample')
        return r['pressureSample']['pressure']


    def get_platform_target_temperature(self):
        r = self.get_prop('/controller/properties/platformTargetTemperature')
        return r['platformTargetTemperature']

    def set_platform_target_temperature(self, target):
        return self.set_prop('/controller/properties/platformTargetTemperature', target)

    def get_platform_temperature(self):
        r = self.get_prop('/sampleChamber/temperatureControllers/platform/thermometer/properties/sample')
        return r['sample']['temperatureOK'], r['sample']['temperatureAvg1Sec']

    def get_platform_temperature_stability(self):
        r = self.get_prop('/sampleChamber/temperatureControllers/platform/thermometer/properties/sample')
        return r['sample']['temperatureStabilityOK'], r['sample']['temperatureStability']

    def set_platform_stability_target(self, target):
        return self.set_prop('/sampleChamber/temperatureControllers/platform/thermometer/properties/stabilityTarget', target)

    def get_platform_temperature_stable(self):
        r = self.get_prop('/sampleChamber/temperatureControllers/platform/thermometer/properties/sample')
        return r['sample']['temperatureStabilityOK'], r['sample']['temperatureStable']

    def get_platform_temperature_sample(self):
        r = self.get_prop('/sampleChamber/temperatureControllers/platform/thermometer/properties/sample')
        return r['sample']


    # User 1
    def set_user1_temperature_controller_enabled(self, enabled):
        return self.set_prop('/sampleChamber/temperatureControllers/user1/properties/controllerEnabled', enabled)
    
    def get_user1_target_temperature(self):
        r = self.get_prop('/sampleChamber/temperatureControllers/user1/properties/targetTemperature')
        return r['targetTemperature']
    
    def set_user1_target_temperature(self, target):
        return self.set_prop('/sampleChamber/temperatureControllers/user1/properties/targetTemperature', target)

    def get_user1_temperature(self):
        r = self.user1_temperature_sample()
        return r['sample']['temperatureOK'], r['sample']['temperatureAvg1Sec']

    def get_user1_temperature_stable(self):
        r = self.get_prop('/sampleChamber/temperatureControllers/user1/thermometer/properties/sample')
        return r['sample']['temperatureStabilityOK'], r['sample']['temperatureStable']

    def get_user1_temperature_stability(self):
        r = self.get_prop('/sampleChamber/temperatureControllers/user1/thermometer/properties/sample')
        return r['sample']['temperatureStabilityOK'], r['sample']['temperatureStability']

    def get_user1_temperature_sample(self):
        r = self.get_prop('/sampleChamber/temperatureControllers/user1/thermometer/properties/sample')
        return r['sample']

    def set_user1_stability_target(self, target):
        return self.set_prop('/sampleChamber/temperatureControllers/user1/thermometer/properties/stabilityTarget', target)


    # User 2
    def set_user2_temperature_controller_enabled(self, enabled):
        return self.set_prop('/sampleChamber/temperatureControllers/user2/properties/controllerEnabled', enabled)

    def set_user2_target_temperature(self, target):
        return self.set_prop('/sampleChamber/temperatureControllers/user2/properties/targetTemperature', target)

    def get_user2_temperature(self):
        r = self.user2_temperature_sample()
        return r['sample']['temperatureOK'], r['sample']['temperatureAvg1Sec']

    def get_user2_temperature_stable(self):
        r = self.get_prop('/sampleChamber/temperatureControllers/user2/thermometer/properties/sample')
        return r['sample']['temperatureStabilityOK'], r['sample']['temperatureStable']

    def get_user2_temperature_stability(self):
        r = self.get_prop('/sampleChamber/temperatureControllers/user2/thermometer/properties/sample')
        return r['sample']['temperatureStabilityOK'], r['sample']['temperatureStability']

    def get_user2_temperature_sample(self):
        r = self.get_prop('/sampleChamber/temperatureControllers/user2/thermometer/properties/sample')
        return r['sample']

    def set_user2_stability_target(self, target):
        return self.set_prop('/sampleChamber/temperatureControllers/user2/thermometer/properties/stabilityTarget', target)



#!/usr/bin/env python3

"""Create an SSH tunnel for Scripting R3

The basic purpose of this is to run something that logically is
   ssh -N -L <local_port>:127.0.0.1:<remote_port> <remote_user>@<remote_host>
Internally it is implemented using sshtunnel.  See 
  https://github.com/pahaz/sshtunnel

"""

import sys
import os 
import subprocess
import time
import signal
import socket
import sshtunnel


class catch_sigint(object):
    def __init__(self):
        self.caught_sigint = False
    def note_sigint(self, signum, frame):
        self.caught_sigint = True
    def __enter__(self):
        self.oldsigint = signal.signal(signal.SIGINT, self.note_sigint)
        return self
    def __exit__(self, *args):
        signal.signal(signal.SIGINT, self.oldsigint)
    def __call__(self):
        return self.caught_sigint

class TunnelError(Exception):
    """Represent general SSH tunnel errors
    """
    def __init__(self, message):
        self.message = message
        return
    def __str__(self):
        return self.message
    
    
def tunnel(remote_host, remote_ports, remote_user="tunnel", local_ports=None):
    """Create an SSH tunnel

    Note- A background thread is implicitly created to handle the tunnel.

    Keyword Parameters: 
       remote_host   The name or IP number of the ECA system.
       remote_ports  A TCP port number (or a list)
       remote_user   The name of the remote user.  Defaults to "tunnel"
       local_ports   The local port to use (or list of ports).
                     A random port is choosen if local_port is None.

    Return: 
       server   The object that created the tunnel.  To get the 
                local port numbers look at
                server.local_bind_ports
    """ 

    def to_port_number(p):
        assert p is not None
        if isinstance(p, str):
            try:
                p = socket.getservbyname(p)
            except:
                p = int(p)
        return p
    
    if not isinstance(remote_ports, (list,tuple)): remote_ports = (remote_ports,)
    if isinstance(remote_ports, (list,)):          remote_ports = tuple(remote_ports)

    if local_ports is None:                       local_ports = (0,) * len(remote_ports)
    if not isinstance(local_ports, (list,tuple)): local_ports = (local_ports,)
    if isinstance(local_ports, (list,)):          local_ports = tuple(local_ports)

    assert len(local_ports) == len(remote_ports), "Must be a one to one mapping from local to remote"

    remote_binds = [('127.0.0.1', to_port_number(p)) for p in remote_ports]
    local_binds  = [('127.0.0.1', to_port_number(p)) for p in local_ports]

    
    pkey   = os.path.join(os.path.join(os.path.expanduser("~"), '.ssh'),
                          'id_rsa')
    server = sshtunnel.SSHTunnelForwarder(
        str(socket.gethostbyname(remote_host)),
        ssh_username          = remote_user,
        ssh_pkey              = pkey,
        local_bind_addresses  = local_binds,
        remote_bind_addresses = remote_binds,
        ssh_port              = 2222)
    try: 
        server.start()
    except sshtunnel.BaseSSHTunnelForwarderError as exc:
        raise TunnelError("Could not start tunnel.  Have you copied your ssh keys?") from None
    return server
    

def _main():
    running = True
    with catch_sigint() as got_sigint:
        ports  = (22, 2222, 2224, 5900, 47101, 47103, 47104)
        local  = None
        server = tunnel('192.168.45.115', ports, remote_user='tunnel', local_ports=local)
        print("started")
        print(f"remote->local ports are {[(ports[i], server.local_bind_ports[i]) for i in range(len(ports))]}")
        while running:
            try:
                print("sleeping")
                time.sleep(1)
                if got_sigint():
                    running = False
                    print("Caught Ctrl-C")
            except IOError:
                print("caught IOError")
                time.sleep(1)
                pass
            except KeyboardInterrupt:
                print("caught KeyboardInterrupt")
                time.sleep(1)
                pass
        try:
            server.stop()
            sys.exit(0)
        except:
            print("Caught exception trying to stop server")
            print(f"{sys.exc_info()}")
            sys.exit(1); 
    
if __name__=="__main__":
    _main()
            
                
        
            
                   



#!/usr/bin/env python3
#

import sys
import os
from pymodaq_plugins_montana.hardware import instrument
import time
import datetime


class Rook(instrument.Instrument):
    def __init__(self, ip, version='v1', verbose=False, tunnel=False, port=instrument.Rest_Ports.lynx_hlm):
        super().__init__(ip=ip,
                         port=port,
                         version=version,
                         verbose=verbose,
                         tunnel=tunnel)

    #
    # Controller commands
    #
    def emergency_stop(self):
        '''Stop all axes'''
        self.call_method(f'controller/methods/emergencyStop()')
        return

    #
    # per-Axis commands
    #
    def get_axis_status(self, stack_num, axis_num):
        """Get the real-time status of the axis"""
        r = self.get_prop(f'stacks/stack{stack_num}/axes/axis{axis_num}/properties/status')
        return r['status']

    def get_axis_moving(self, stack_num, axis_num):
        return self.get_axis_status(stack_num, axis_num)['moving']

    def get_axis_target_position(self, stack_num, axis_num):
        '''Get the target position for the given axis.'''
        return self.get_axis_status(stack_num, axis_num)['targetPosition']

    def get_axis_encoder_position(self, stack_num, axis_num):
        '''Get the encoder position for the given axis.'''
        return self.get_axis_status(stack_num, axis_num)['encoderPosition']

    def wait_for_axis_not_moving(self, stack_num, axis_num, max_wait=datetime.timedelta(seconds=60)):
        """Returns True if the axis reached the target before timeout, False
           if timeout occurred and axis was still moving."""

        reached_target = False
        start_time  = datetime.datetime.now()
        while not reached_target:
            # Delay to give time for last move command to begin and
            # state to propogate through system
            time.sleep(0.5)

            reached_target = not self.get_axis_moving(stack_num, axis_num)

            # Break if we've exceeded the max_wait
            elapsed_time = datetime.datetime.now() - start_time
            if not reached_target and elapsed_time > max_wait:
                self.stop_axis(stack_num, axis_num)
                print(f'Timed out before reaching target, stopped axis {axis_num}.')
                break

        return reached_target

    def set_axis_velocity(self, stack_num: int, axis_num: int, velocity: float):
        '''Set the target velocity of the given axis

        velocity in m/s
        '''
        self.set_prop(f'stacks/stack{stack_num}/axes/axis{axis_num}/properties/velocity', velocity)
        return

    def get_axis_velocity(self, setack_num, axis_num) -> float:
        return self.get_prop(f'stacks/stack{stack_num}/axes/axis{axis_num}/properties/velocity')['velocity']

    def move_axis_to_negative_limit(self, stack_num, axis_num, wait=False):
        '''Move axis to the negative limit'''
        self.call_method(f'stacks/stack{stack_num}/axes/axis{axis_num}/methods/moveToNegativeLimit()')
        if wait: self.wait_for_axis_not_moving(stack_num, axis_num)
        return

    def move_axis_to_positive_limit(self, stack_num, axis_num, wait=False):
        '''Move axis to the positive limit'''
        self.call_method(f'stacks/stack{stack_num}/axes/axis{axis_num}/methods/moveToPositiveLimit()')
        if wait: self.wait_for_axis_not_moving(stack_num, axis_num)
        return

    def move_axis_absolute_position(self, stack_num, axis_num, pos, wait=False):
        """Move axis to an absolute position

        Parameter
        ---------
        pos: float
            position in meter
        """
        self.call_method(f'stacks/stack{stack_num}/axes/axis{axis_num}/methods/moveAbsolute(double:pos)', pos)
        if wait: self.wait_for_axis_not_moving(stack_num, axis_num)
        return

    def jog_axis(self, stack_num, axis_num, move_positive_direction):
        '''Jog axis continuously in one direction with no target position'''
        direction = 'Positive' if move_positive_direction else 'Negative'
        self.call_method(f'stacks/stack{stack_num}/axes/axis{axis_num}/methods/jog(JogDirection:dir)', direction)
        return

    def stop_axis(self, stack_num, axis_num):
        '''Stop an axis'''
        self.call_method(f'stacks/stack{stack_num}/axes/axis{axis_num}/methods/stop()')
        return

    def zero_axis(self, stack_num, axis_num):
        '''Zero the encoder position of an axis'''
        self.call_method(f'stacks/stack{stack_num}/axes/axis{axis_num}/methods/zero()')
        return

    def set_axis_closed_loop(self, stack_num, axis_num, cl_enabled):
        '''Enable/disable closed loop feedback mode'''
        direction = 'ClosedLoop' if cl_enabled else 'OpenLoop'
        self.set_prop(f'stacks/stack{stack_num}/axes/axis{axis_num}/properties/feedbackMode', direction)
        return

    def set_axis_closed_loop_deadband_counts(self, stack_num, axis_num, counts):
        '''Set the closed-loop deadband encoder counts'''
        self.set_prop(f'stacks/stack{stack_num}/axes/axis{axis_num}/properties/closedLoopDeadbandCounts', counts)
        return

    def set_axis_closed_loop_deadband_timeout(self, stack_num, axis_num, timeout):
        '''Set the closed-loop deadband encoder timeout'''
        self.set_prop(f'stacks/stack{stack_num}/axes/axis{axis_num}/properties/closedLoopDeadbandTimeout', timeout)
        return
    
    def set_axis_hard_stop_detection_enabled(self, stack_num, axis_num, enabled):
        '''Enable/disable hard stop detection'''
        self.set_prop(f'stacks/stack{stack_num}/axes/axis{axis_num}/properties/hardStopDetectionEnabled', enabled)
        return
        
    def set_axis_hard_stop_rebound_distance(self, stack_num, axis_num, distance):
        '''Set the hardstop rebound distance'''
        self.set_prop(f'stacks/stack{stack_num}/axes/axis{axis_num}/properties/hardStopReboundDistance', distance)
        return
        
    def set_axis_hard_stop_sensitivity(self, stack_num, axis_num, sensitivity):
        '''Set the hardstop detection sensitivity'''
        self.set_prop(f'stacks/stack{stack_num}/axes/axis{axis_num}/properties/hardStopSensitivity', sensitivity)
        return


if __name__ == "__main__":

    rook = Rook('10.35.3.38', verbose=False)

    stack_num = 1
    axis_num  = 1

    for p in [0, 100, 200, 300, 400, 500, 0]: # um
        print(f'Move absolute to {p}')
        rook.move_axis_absolute_position(stack_num, axis_num, p/1000/1000)
        if rook.wait_for_axis_not_moving(stack_num, axis_num):
            print('Target acquired.')
        else:
            print('Failed to acquire target before timeout occurred.')



# -*- coding: utf-8 -*-
"""
Created the 17/10/2023

@author: Sebastien Weber
"""
import pytest
from pathlib import Path
import importlib
import pkgutil


MANDATORY_MOVE_METHODS = ['ini_attributes', 'get_actuator_value', 'close', 'commit_settings',
                          'ini_stage', 'move_abs', 'move_home', 'move_rel', 'stop_motion']
MANDATORY_VIEWER_METHODS = ['ini_attributes', 'grab_data', 'close', 'commit_settings',
                          'ini_detector', ]


def get_package_name():
    here = Path(__file__).parent
    package_name = here.parent.stem
    return package_name

def get_move_plugins():
    pkg_name = get_package_name()
    try:
        move_mod = importlib.import_module(f'{pkg_name}.daq_move_plugins')
        plugin_list = [mod for mod in [mod[1] for mod in
                                       pkgutil.iter_modules([str(move_mod.path.parent)])]
                       if 'daq_move_' in mod]
    except ModuleNotFoundError:
        plugin_list = []
        move_mod = None
    return plugin_list, move_mod


def get_viewer_plugins(dim='0D'):
    pkg_name = get_package_name()
    try:
        viewer_mod = importlib.import_module(f'{pkg_name}.daq_viewer_plugins.plugins_{dim}')

        plugin_list = [mod for mod in [mod[1] for mod in
                                       pkgutil.iter_modules([str(viewer_mod.path.parent)])]
                       if f'daq_{dim}viewer_' in mod]
    except ModuleNotFoundError:
        plugin_list = []
        viewer_mod = None
    return plugin_list, viewer_mod


def test_package_name_ok():
    assert 'pymodaq_plugins_' in get_package_name()[0:16]


def test_imports():
    pkg_name = get_package_name()
    mod = importlib.import_module(pkg_name)
    assert hasattr(mod, 'config')
    assert hasattr(mod, '__version__')
    move_mod = importlib.import_module(f'{pkg_name}', 'daq_move_plugins')
    importlib.import_module(f'{pkg_name}', 'daq_viewer_plugins')
    importlib.import_module(f'{pkg_name}', 'extensions')
    importlib.import_module(f'{pkg_name}', 'models')
    importlib.import_module(f'{pkg_name}.daq_viewer_plugins', 'plugins_0D')
    importlib.import_module(f'{pkg_name}.daq_viewer_plugins', 'plugins_1D')
    importlib.import_module(f'{pkg_name}.daq_viewer_plugins', 'plugins_2D')
    importlib.import_module(f'{pkg_name}.daq_viewer_plugins', 'plugins_ND')


def test_move_inst_plugins_name():
    plugin_list, move_mod = get_move_plugins()
    for plug in plugin_list:
        name = plug.split('daq_move_')[1]
        assert hasattr(getattr(move_mod, plug), f'DAQ_Move_{name}')


def test_move_has_mandatory_methods():
    plugin_list, move_mod = get_move_plugins()
    for plug in plugin_list:
        name = plug.split('daq_move_')[1]
        klass = getattr(getattr(move_mod, plug), f'DAQ_Move_{name}')
        for meth in MANDATORY_MOVE_METHODS:
            assert hasattr(klass, meth)


@pytest.mark.parametrize('dim', ('0D', '1D', '2D', 'ND'))
def test_viewer_has_mandatory_methods(dim):
    plugin_list, mod = get_viewer_plugins(dim)
    for plug in plugin_list:
        name = plug.split(f'daq_{dim}viewer_')[1]
        try:
            module = importlib.import_module(f'.{plug}', mod.__package__)
        except Exception:
            break
        klass = getattr(module, f'DAQ_{dim}Viewer_{name}')
        for meth in MANDATORY_VIEWER_METHODS:
            assert hasattr(klass, meth)



#!/usr/bin/env python3

"""Helper code for retrieving information from the remote support server."""

import sys
import os
import requests
import http
import json
import enum

_mirs_host = 'mirs'

class PrintUsage(Exception):
    def __init__(self):
        return super().__init__()        


class MalformedFilename(Exception):
    def __init__(self, path):
        super().__init__()
        self.path = path
        return
    def __str__(self):
        return f"{super().__str__()} Malformed path: {self.path}"

class TooManyHosts(Exception):
    def __init__(self):
        return super().__init__()        
    def __str__(self):
        return f"You may only specify one host."

class TooManyUsers(Exception):
    def __init__(self):
        return super().__init__()        
    def __str__(self):
        return f"You may only specify one user account."


class UnsupportedUsers(Exception):
    def __init__(self, allowed):
        super().__init__()
        self.allowed = allowed
    def __str__(self):
        return f"Only the following users are supported: {self.allowed}."


class UnspecifiedHost(Exception):
    def __init__(self):
        return super().__init__()        
    def __str__(self):
        return f"You must a host for the copy."


    
class MirsClientNotFound(Exception):
    def __init__(self, miec):
        super().__init__()
        self.miec = miec
        return
    def __str__(self):
        return f"{super().__str__()} Unknown system {self.miec}"

def all_tunnels(mirs_server=_mirs_host, rest_port=8080):
    """
    Retrieve a list of all tunnels.

    Return
        Dictionary of tunnels.  The key of each item is the name of
        the system.  The data in each item is itself a dictionary with
        the miec port number as the key, the mirs port number as the
        data.  Example:
          {'CR-713':      {'47101': 40861, 
                           '2222':  52389}, 
           'Cryostation': {'22':   45861, 
                           '2222': 49861, 
                           '5900': 50861}}
    """
    result = json.loads(requests.get(f'http://{mirs_server}:{rest_port}/tunnels').content.decode('utf-8'))
    return result;

def all_ips(mirs_server=_mirs_host, rest_port=8080):
    """
    Retrieve a list of each system and its IP address

    Return
        Dictionary of system:ip values
        Example:
          {'CR-732': '128.174.165.72', 
           'CR-801': '206.196.177.114',
           ...}

    """
    result = json.loads(requests.get(f'http://{mirs_server}:{rest_port}/ips').content.decode('utf-8'))
    return result;

def lookup_port(miec, miec_port, mirs_server=_mirs_host, rest_port=8080):
    """Lookup port on mirs server for tunneling into the MIEC.
    
    Keyword Parameters
      miec         The name of the MIEC to attach to (e.g. CR-712)
      miec_port    Port on the MIEC to attach to 
      mirs_server  The remote support server.  (i.e. the computer running 
                   tunnel_display.)
      rest_port    Port number
    """
    if isinstance(miec_port, enum.Enum): miec_port = miec_port.value
    tunnels = all_tunnels()
    try:
        if miec not in tunnels:
            for k in tunnels.keys(): miec = k if k.lower()==miec.lower() else miec                
        return tunnels[miec][str(miec_port)]
    except KeyError:
        pass
    raise MirsClientNotFound(miec)

def rewrite_ssh_args(argv, mirs_server=_mirs_host):
    """Rewrite the ssh arguments to run through the MIRS tunnel. 
    """
    cmd = os.path.basename(argv[0])

    if cmd=='mirs-scp':
        flags   = ['-3', '-4', '-6', '-B', '-C', '-p', '-q', '-r', '-v']
        options = ['-c cipher', '-F ssh_config', '-i identity_file',
                   '-l limit', '-o ssh_option', '-P port', '-S program']
        result = ['scp', '-o' 'StrictHostKeyChecking=no']
    else:
        flags   = ["-4", "-6", "-A", "-a", "-C", "-f", "-G", "-g", "-h", "-K",
                   "-k", "-M", "-N", "-n", "-q", "-s", "-T", "-t", "-V", "-v",
                   "-X", "-x", "-Y", "-y"]
        options = ["-b", "-c", "-D", "-E", "-e", "-F", "-I", "-i", "-J", "-L",
                   "-l", "-m", "-O", "-o", "-p", "-Q", "-R", "-S", "-W", "-w"]
        result = ['ssh', '-o' 'StrictHostKeyChecking=no']

    i = 1
    while i<len(argv):
        if argv[i] in flags or argv[i]=='--help':
            result.append(argv[i])
            if argv[i] in ('-h', '--help'): raise PrintUsage()
            if argv[i]=='-3':
                sys.stderr.write("Error: 3-way copies are not supported\n")
                raise PrintUsage()
            if argv[i] in ('-p', '-P'):
                sys.stderr.write("Error: Specifying an alternate port is not supported\n")
                raise PrintUsage()
        elif argv[i] in options:
            result.append(argv[i])
            i = i+1
            result.append(argv[i])
        elif argv[i][:2] in options:
            result.append(argv[i])
        else:
            #
            # We've hit the end of the flags and options.  For scp argv[i] is
            # the name of the host.  The arguments after that (if any)
            # are the name of a command and its command line arguments.
            #
            # For scp the remaining arguments have hostnames and need
            # to be rewritting accordingly
            #
            if result[0]=='ssh':
                if '-6' not in argv: result.append('-4')
                result.append('-p')
                result.append(f'{lookup_port(argv[i], 2222, mirs_server)}')
                result.append(f'mi@{mirs_server}')
                result = result + argv[i+1:]
            else:                    
                def _split_scp_fname(fname):
                    user = ''
                    host = ''
                    if fname.find('@') > fname.find(':'):           raise MalformedFilename(fname)
                    if fname.find('@')!=-1 and fname.find(':')==-1: raise MalformedFilename(fname)
                    if fname.find('@')!=-1:
                        user  = fname[:fname.find('@')] + '@'
                        fname = fname[fname.find('@')+1:]
                    if fname.find(':')!=-1:
                        host  = fname[:fname.find(':')] + ':'
                        fname = fname[fname.find(':')+1:]
                    result = [user, host, fname]
                    return result
                def _find_scp_port(fnames):
                    hosts = [h for h in [_split_scp_fname(f)[1] for f in fnames] if h]
                    if len(hosts)==0: raise UnspecifiedHost()
                    if len(hosts)>1:  raise TooManyHosts()
                    return lookup_port(hosts[0][:-1], 2222, mirs_server)
                def _find_scp_user(fnames):
                    users = [u for u in list({_split_scp_fname(f)[0]:0 for f in fnames}) if u]
                    if len(users)==0: users = ['mi@']
                    if len(users)>1:  raise TooManyUsers()
                    return users[0][:-1]
                def _convert_scp_filenames(fnames, user):
                    altered_names = []
                    for f in fnames: 
                        tmp = _split_scp_fname(f)
                        if len(tmp[1])>0:
                            tmp[1] = mirs_server + ':'
                            tmp[0] = user + '@'
                        altered_names.append(''.join(tmp))
                    return altered_names

                fnames = argv[i:]
                result.append('-P')
                result.append(str(_find_scp_port(fnames)))
                              
                result = result + _convert_scp_filenames(fnames, _find_scp_user(fnames))
            break
        i = i+1
    return result

if __name__ == "__main__":
    print(f"all_tunnels is {all_tunnels(_mirs_host)}")
    print(f"all_ips is {all_ips(_mirs_host)}")
    
    



#!/usr/bin/env python3
#
"""
Class to simplify RESTful API communication with the CryoCore instrument.

Example usage:
 import cryocore
 cryo = cryocore.CryoCore('192.168.1.123')

 # Shows how to use high-level methods for accessing common functions
 cryo.cooldown()
 cryo.set_platform_target_temperature(3.1)
 cryo.get_platform_temperature()
 cryo.get_platform_temperature_stability()
 cryo.warmup()

 # Shows how to use generic post/get/put methods for accessing any REST url/end-point
 cryo.call_method('/controller/methods/cooldown()')
 cryo.get_prop('/sampleChamber/temperatureControllers/platform/thermometer/sample')['sample']['temperatureAvg1Sec']
 cryo.set_prop('/controller/properties/platformTargetTemperature', 1.7)
"""
import sys
import os
from . import instrument
from . import genericcryostat

Ports = instrument.Rest_Ports


@genericcryostat.register
class CryoCore(genericcryostat.GenericCryostat):
    def __init__(self, ip, version='v1', verbose=False, tunnel=False, port=Ports.cryocore_hlm):
        super().__init__(ip=ip,
                         port=port,
                         version=version,
                         verbose=verbose,
                         tunnel=tunnel)

    def cooldown(self):
        self.call_method('/controller/methods/cooldown()')

    def warmup(self):
        self.call_method('/controller/methods/warmup()')

    def vent(self):
        self.call_method('/controller/methods/vent()')

    def pull_vacuum(self):
        self.call_method('/controller/methods/pullVacuum()')

    def abort_goal(self):
        self.call_method('/controller/methods/abortGoal()')

    def get_system_goal(self):
        return self.get_prop('/controller/properties/systemGoal')['systemGoal']

    def get_system_state(self):
        return self.get_prop('/controller/properties/systemState')['systemState']

    def get_sample_chamber_pressure(self):
        return self.get_prop('/vacuumSystem/vacuumGauges/sampleChamberPressure/properties/pressureSample')['pressureSample']['pressure']

    def set_platform_bakeout_enabled(self, enabled):
        return self.set_prop('/controller/properties/platformBakeoutEnabled', enabled)

    def set_platform_bakeout_temperature(self, temperature):
        return self.set_prop('/controller/properties/platformBakeoutTemperature', temperature)

    def set_platform_bakeout_time(self, duration):
        return self.set_prop('/controller/properties/platformBakeoutTime', duration)

    def set_dry_nitrogen_purge_enabled(self, enabled):
        return self.set_prop('/controller/properties/dryNitrogenPurgeEnabled', enabled)

    def set_dry_nitrogen_purge_num_times(self, times):
        return self.set_prop('/controller/properties/dryNitrogenPurgeNumTimes', times)

    #
    # Vent methods
    #
    def set_vent_continuously_enabled(self, enabled):
        return self.set_prop('/controller/properties/ventContinuouslyEnabled', enabled)

    #
    # Pull Vacuum methods
    #
    def set_pull_vacuum_target_pressure(self, target):
        return self.set_prop('/controller/properties/pullVacuumTargetPressure', target)

    #
    # Stage1 methods
    #
    def get_stage1_temperature_sample(self):
        r = self.get_prop('/cooler/temperatureControllers/stage1/thermometer/properties/sample')
        return r['sample']

    def get_stage1_temperature(self):
        r = self.get_prop('/cooler/temperatureControllers/stage1/thermometer/properties/sample')
        return r['sample']['temperatureOK'], r['sample']['temperatureAvg1Sec']


    #
    # Stage2 methods
    #
    def get_stage2_temperature_sample(self):
        r = self.get_prop('/cooler/temperatureControllers/stage2/thermometer/properties/sample')
        return r['sample']

    def get_stage2_temperature(self):
        r = self.get_prop('/cooler/temperatureControllers/stage2/thermometer/properties/sample')
        return r['sample']['temperatureOK'], r['sample']['temperatureAvg1Sec']

    #
    # Platform methods
    #
    def get_platform_target_temperature(self):
        r = self.get_prop('/controller/properties/platformTargetTemperature')
        return r['platformTargetTemperature']

    def set_platform_target_temperature(self, target):
        return self.set_prop('/controller/properties/platformTargetTemperature', target)

    def get_platform_temperature(self):
        r = self.get_prop('/sampleChamber/temperatureControllers/platform/thermometer/properties/sample')
        return r['sample']['temperatureOK'], r['sample']['temperatureAvg1Sec']

    def get_platform_temperature_stability(self):
        r = self.get_prop('/sampleChamber/temperatureControllers/platform/thermometer/properties/sample')
        return r['sample']['temperatureStabilityOK'], r['sample']['temperatureStability']

    def set_platform_stability_target(self, target):
        return self.set_prop('/sampleChamber/temperatureControllers/platform/thermometer/properties/stabilityTarget', target)

    def get_platform_temperature_stable(self):
        r = self.get_prop('/sampleChamber/temperatureControllers/platform/thermometer/properties/sample')
        return r['sample']['temperatureStabilityOK'], r['sample']['temperatureStable']

    def get_platform_temperature_sample(self):
        r = self.get_prop('/sampleChamber/temperatureControllers/platform/thermometer/properties/sample')
        return r['sample']

    def get_platform_heater_sample(self):
        r = self.get_prop('/sampleChamber/temperatureControllers/platform/heater/properties/sample')
        return r['sample']

    #
    # User1 methods
    #
    def get_user1_temperature_sample(self):
        r = self.get_prop('/sampleChamber/temperatureControllers/user1/thermometer/properties/sample')
        return r['sample']

    def get_user1_temperature(self):
        r = self.get_prop('/sampleChamber/temperatureControllers/user1/thermometer/properties/sample')
        return r['sample']['temperatureOK'], r['sample']['temperatureAvg1Sec']

    def get_user1_temperature_stability(self):
        r = self.get_prop('/sampleChamber/temperatureControllers/user1/thermometer/properties/sample')
        return r['sample']['temperatureStabilityOK'], r['sample']['temperatureStability']
    
    #
    # Turbopump
    #
    def set_turbopump_precooling_target_pressure(self, target):
        return
    
    def set_turbopump_precooling_time_limit(self, limit):
        return



#!/usr/bin/env python3


"""Base class forREST API on MI cryostations
"""

import sys
import os
from . import instrument

from typing import Optional, List, Tuple, Any

class PidScheduleItem:
    def __init__(self, temperature, kc=None, ti=None, td=None):
        self.temperature = temperature
        self.kc          = kc
        self.ti          = ti
        self.td          = td
        return
    def __lt__(self, other): 
        if self.temperature != other.temperature: return self.temperature < other.temperature
        elif self.kc != other.kc:                 return self.kc < other.kc
        elif self.ti != other.ti:                 return self.ti < other.ti
        else:                                     return self.td < other.td
    def __repr__(self):
        return repr((self.temperature, self.kc, self.ti, self.td))

    
class GenericCryostat(instrument.Instrument):
    """Functionality common to all cryostat instruments. 

    All derived classes should register themselves using the
    @genericcryostat.register decorator. 
    """
    def __init__(self, ip, port, version='v1', verbose=False, tunnel=False):
        super().__init__(ip=ip,
                         port=port,
                         version=version,
                         verbose=verbose,
                         tunnel=tunnel)
        return
    
    def cryo_thermometer_channels(self) -> List[tuple]:
        """Return names of the thermometers for s and xp series Cryostations

        Return 
           tuple(location, channel) 
           example ('sampleChamber', 'platform')
        """

        def is_tc(x):
            """Determine if endpoint is a TC (temperature controller)

            We use this to filter out all the non-TC endpoints when we
            iterate through the list of all endpoints.
            """
            return (len(x)==10
                    and x[3]=='v1'
                    # and x[4] in ('cooler', 'sampleChamber', 'magnet')
                    and x[5]=='temperatureControllers'
                    and x[7:10]==['thermometer', 'properties', 'sample'])
            
        # First we'll search for endpoints match the following pattern
        #   0    1       2             3           4                              5            6      7          8        9
        #  http://192.168.45.184:47103/v1/{cooler|sampleChamber|magnet}/temperatureControllers/*/thermometer/properties/sample
        return list(set([(x[4], x[6]) for x in [x.split('/') for x in self.get('/')] if is_tc(x)]))

    def cryo_thermometer_channels_by_name(self) -> dict:
        """Return a Lookup table from name of the channel to return a
           fully unique description (i.e. the channel name and
           location) for example
              {'platform',   ('sampleChamber', 'platform'), 
               'stage1',     ('cooler',        'stage1')}
        """
        return { x[1]: x for x in self.cryo_thermometer_channels() }
    
    def cryo_tc_channel_endpoint_root(self, channel):
        """Return endpoint root for TCs for s and xp series Cryostations

        This is designed to use in conjunction with instrument.get_prop()

        Keywords arguments:
           channel     A pair of strings.  First item is the name of the 
                       chamber (e.g. cooler).  Second item is the channel
                       (e.g. stage2)
        """

        return f'/{channel[0]}/temperatureControllers/{channel[1]}'
    
    def _tc_root(self, channel): return self.cryo_tc_channel_endpoint_root(channel)

    def cryo_tc_pid_schedule_endpoint(self, channel):
        """Return endpoint name of the PID schedule for a cryostation

        This is designed to use in conjunction with instrument.get_prop()

        Keywords arguments:
           channel     A pair of strings.  First item is the name of the 
                       chamber (e.g. cooler).  Second item is the channel
                       (e.g. stage2)
        """

        return f'{self.cryo_tc_channel_endpoint_root(channel)}/properties/pidSchedule'


    def cryo_thermometer_channel_sample_endpoint(self, channel):
        """Return endpoint names of the thermometers for s and xp series Cryostations

        This is designed to use in conjunction with instrument.get_prop()

        Keywords arguments:
           channel     A pair of strings.  First item is the name of the 
                       chamber (e.g. cooler).  Second item is the channel
                       (e.g. stage2)
        """

        return f'{self.cryo_tc_channel_endpoint_root(channel)}/thermometer/properties/sample'

    def cryo_heater_channel_sample_endpoint(self, channel):
        """Return endpoint names of the heaters for s and xp series Cryostations

        This is designed to use in conjunction with instrument.get_prop()

        Keywords arguments:
           channel     A pair of strings.  First item is the name of the 
                       chamber (e.g. cooler).  Second item is the channel
                       (e.g. stage2)
        """

        return f'{self.cryo_tc_channel_endpoint_root(channel)}/heater/properties/sample'

    def cryo_onoff_channel_sample_endpoint(self, channel):
        """Return endpoint names of the thermometers for s and xp series Cryostations

        This is designed to use in conjunction with instrument.get_prop()

        Keywords arguments:
           channel     A pair of strings.  First item is the name of the 
                       chamber (e.g. cooler).  Second item is the channel
                       (e.g. stage2)
        """

        return f'{self.cryo_tc_channel_endpoint_root(channel)}/properties/onOffSample'

    def set_on_off_power(self, channel: tuple, value: float) -> None:
        path = self._tc_root(channel) + "/properties/onOffPower"
        return self.set_prop(path, {'onOffPower': value})
    

    def set_on_off_hysteresis(self, channel: int, value: float) -> None:
        path = self._tc_root(channel) + "/properties/onOffHysteresis"
        return self.set_prop(path, {'onOffHysteresis': value})
        
    def enable_controller(self, channel, mode, temperature):
        assert mode in ('OnOff', 'PID')
        root = self.cryo_tc_channel_endpoint_root(channel)
        return self.call_method(root + "/methods/enableController(ControlMode:mode,double:temperature)",
                         {'mode':         mode,
                          'temperature':  temperature})

    def disable_controller(self, channel):
        return self.call_method(self.cryo_tc_channel_endpoint_root(channel)
                         + "/methods/disableController()")

    def get_tc_pid_schedule(self, channel: Tuple[str,str]) -> List[PidScheduleItem]:
        """Return the PID schedule for the temperature controller. 

        Keywords arguments:
           channel     A pair of strings.  First item is the name of the 
                       chamber (e.g. sampleChamber).  Second item is the channel
                       (e.g. platform)
        """
        ep = self.cryo_tc_pid_schedule_endpoint(channel)
        print(f"ep is {ep}")
        try:
            sched = self.get_prop(ep)
        except instrument.ApiError:
            print("Failed to get pid schedule")
            sched = {'pidSchedule': {'rows': []}}
        return sorted([PidScheduleItem(temperature = row['temperature'],
                                       kc          = row['kc'],
                                       ti          = row['ti'],
                                       td          = row['td']) 
                       for row in sched['pidSchedule']['rows']])
    
    def set_tc_pid_schedule(self,
                            channel: Tuple[str,str],
                            sched:   List[PidScheduleItem]) -> None:
        """Store the PID schedule on the system. 
        """
        data = {'rows': [{'temperature': r.temperature, 'kc': r.kc, 'ti': r.ti, 'td': r.td}
                                         for r in sched]}
        print(f"data is {data}")
        print(f"Trying to set |{self.cryo_tc_pid_schedule_endpoint(channel)}|")
        self.set_prop(self.cryo_tc_pid_schedule_endpoint(channel), data)
        return 
        

    
_cryo_classes = {}
def register_cryo_class(cls, create=None):
    """

    Keyword arguments: 
       cls     A cryostation class.  (Should be derived from GenericCryostat)
       create  A function to create an object.  
    """
    if create is None: _cryo_classes[cls] = lambda ip, tunnel: cls(ip, 'v1', False, tunnel)
    else:              _cryo_classes[cls] = create
    return

def register(cls):
    """Class decorator to register a class derived from GenericCryostat

    Classes that are registered can take part in automatic device
    discovery through the connect_cryostat() function.

    Example
       @genericcryostat.register
       class SCryostation(genericcryostat.GenericCryostat): 
          ...

    Keyword arguments
      cls    The class being decorated.

    """
    assert issubclass(cls, GenericCryostat), "only cryostat objects should be registered as cryostats"
    register_cryo_class(cls)
    return cls

def connect_cryostat(host: str, tunnel: bool) -> GenericCryostat:
    """
    Try to connect to a cryostation HLM on the given IP address. 

    This function will try to connect to a cryostation using one 
    of the classes derived from GenericCryostat and registered 
    with the @genericcryostat.register class decorator.

    Keyword Paramameters
       host   -- Host name or IP address of a cryostation 
       tunnel -- Communicate through a secure SSH tunnel?

    Return 
       A GenericCryostat derived object, None if no connection could
       be established.         
    """
    #
    # First try to connect using one of the registered cryostat
    # classes.  Remember, this can only find a cryostat class if it's
    # been registered with the @genericcryostat.register decorator or
    # the register_cryo_class() function.
    # 
    cryo = None
    try:
        for c in _cryo_classes.keys():
            func = _cryo_classes[c]    # lookup the function to make a cryostation object
            cryo = func(host, tunnel)
            if cryo.is_up(): break;
            cryo.close()
            cryo = None
    except instrument.BadUrl:
        cryo = None

    #
    # If we couldn't find a specific one, try a generic.  Currently
    # this only looks on two different port numbers.
    #
    try:
        if cryo is None:
            for p in [instrument.Rest_Ports.scryostation_hlm,
                      instrument.Rest_Ports.xpcryostation_hlm]:
                cryo = GenericCryostat(host,
                                       port    = p,
                                       version = 'v1',
                                       tunnel  = tunnel);
                if cryo.is_up(): break
                cryo.close()
                cryo = None
    except instrument.BadUrl:
        cryo = None

    if cryo is None:
        print("raising instrument.CouldNotConnect()")
        raise instrument.CouldNotConnect()
    
    return cryo






from typing import Union, List, Dict
from pymodaq.control_modules.move_utility_classes import (DAQ_Move_base, comon_parameters_fun,
                                                          main, DataActuatorType, DataActuator)

from pymodaq_utils.logger import set_logger, get_module_name
from pymodaq_utils.utils import ThreadCommand  # object used to send info back to the main thread

from pymodaq_data import Q_

from pymodaq_gui.parameter import Parameter

from pymodaq_plugins_montana.hardware.rook import Rook
from pymodaq_plugins_montana.utils import Config as PluginConfig


plugin_config = PluginConfig()
logger = set_logger(get_module_name(__file__))


class DAQ_Move_Rook(DAQ_Move_base):
    """ Instrument plugin class for an actuator.
    
    This object inherits all functionalities to communicate with PyMoDAQ’s DAQ_Move module through inheritance via
    DAQ_Move_base. It makes a bridge between the DAQ_Move module and the Python wrapper of a particular instrument.

    Attributes:
    -----------
    controller: object
        The particular object that allow the communication with the hardware, in general a python wrapper around the
         hardware library.
         

    """
    is_multiaxes = True
    _axis_names: Union[List[str], Dict[str, int]] = {'Xaxis': 1, 'Yaxis': 2, 'Zaxis':3}
    _controller_units: Union[str, List[str]] = 'µm'
    _epsilon: Union[float, List[float]] = 0.1
    data_actuator_type = DataActuatorType.DataActuator
    params = [
                 {'title': 'Server IP:', 'name': 'ipaddress', 'type': 'str', 'value': plugin_config('ip_address')},
                 {'title': 'Cryo. Port:', 'name': 'cryo_port', 'type': 'int', 'value': plugin_config('rook_port')},
                 {'title': 'Close Loop:', 'name': 'close_loop', 'type': 'bool', 'value': True},
                 {'title': 'Velocity:', 'name': 'velocity', 'type': 'float', 'value': 1, 'suffix': 'um/s'},
                ] + comon_parameters_fun(is_multiaxes, axis_names=_axis_names, epsilon=_epsilon)

    def ini_attributes(self):
        self.controller: Rook = None
        self.stack_num = 1

    def get_actuator_value(self):
        """Get the current value from the hardware with scaling conversion.

        Returns
        -------
        float: The position obtained after scaling conversion.
        """

        pos = DataActuator(data=self.controller.get_axis_encoder_position(self.stack_num, self.axis_value),
                           units='m')
        pos = self.get_position_with_scaling(pos)
        return pos

    def user_condition_to_reach_target(self) -> bool:
        """ Implement a condition for exiting the polling mechanism and specifying that the
        target value has been reached

       Returns
        -------
        bool: if True, PyMoDAQ considers the target value has been reached
        """
        return not self.controller.get_axis_moving(self.stack_num, self.axis_value)

    def close(self):
        """Terminate the communication protocol"""
        self.controller.close()

    def commit_settings(self, param: Parameter):
        """Apply the consequences of a change of value in the detector settings

        Parameters
        ----------
        param: Parameter
            A given parameter (within detector_settings) whose value has been changed by the user
        """
        if param.name() == 'close_loop':
            self.controller.set_axis_closed_loop(self.stack_num, self.axis_value,
                                                 param.value())
        elif param.name() == 'velocity':
            self.controller.set_axis_velocity(self.stack_num, self.axis_value,
                                              Q_(param.value(), param.opts['suffix']).m_as('m/s'))

    def ini_stage(self, controller=None):
        """Actuator communication initialization

        Parameters
        ----------
        controller: (object)
            custom object of a PyMoDAQ plugin (Slave case). None if only one actuator by controller (Master case)

        Returns
        -------
        info: str
        initialized: bool
            False if initialization failed otherwise True
        """

        if self.is_master:  # is needed when controller is master
            self.controller = Rook(ip=self.settings['ipaddress'], port=self.settings['cryo_port'])
        else:
            self.controller = controller
        self.commit_settings(self.settings.child('close_loop'))
        self.commit_settings(self.settings.child('velocity'))

        self.emit_status(ThreadCommand('update_ui', attribute='set_abs_value_red',
                                           args=[Q_(0, 'um')]))

        self.emit_status(ThreadCommand('update_ui', attribute='set_abs_value_green',
                                   args=[Q_(10, 'um')]))

        self.emit_status(ThreadCommand('update_ui', attribute='set_abs_value',
                                       args=[Q_(10, 'um')]))

        self.emit_status(ThreadCommand('update_ui', attribute='set_rel_value',
                                       args=[Q_(1, 'um')]))

        info = "Rook is connected"
        initialized = True
        return info, initialized

    def move_abs(self, value: DataActuator):
        """ Move the actuator to the absolute target defined by value

        Parameters
        ----------
        value: (float) value of the absolute target positioning
        """
        value = self.check_bound(value)  #if user checked bounds, the defined bounds are applied here
        self.target_value = value
        value = self.set_position_with_scaling(value)  # apply scaling if the user specified one
        self.controller.move_axis_absolute_position(self.stack_num, self.axis_value,
                                                    value.to_base_units().value())

    def move_rel(self, value: DataActuator):
        """ Move the actuator to the relative target actuator value defined by value

        Parameters
        ----------
        value: (float) value of the relative target positioning
        """
        value = self.check_bound(self.current_position + value) - self.current_position
        self.target_value = value + self.current_position
        value = self.set_position_relative_with_scaling(value)

        self.move_abs(self.target_value)

    def move_home(self):
        """Call the reference method of the controller"""

        self.controller.move_axis_to_negative_limit(self.stack_num, self.axis_value, wait=True)

    def stop_motion(self):
      """Stop the actuator and emits move_done signal"""

      self.controller.stop_axis(self.stack_num, self.axis_value)


if __name__ == '__main__':
    main(__file__)



#!/usr/bin/env python3
#
"""
Class to simplify RESTful API communication with the FSR instrument.

Example usage:
 import fsr
 fsr = fsr.Fsr('192.168.1.123')

 # Shows how to use high-level methods for accessing common functions
 fsr.cooldown()
 fsr.set_platform_target_temperature(3.1)
 fsr.get_platform_temperature()
 fsr.get_platform_temperature_stability()
 fsr.warmup()

 # Shows how to use generic post/get/put methods for accessing any REST url/end-point
 fsr.call_method('/controller/methods/cooldown()')
 fsr.get_prop('/sampleChamber/temperatureControllers/platform/thermometer/sample')['sample']['temperatureAvg1Sec']
 fsr.set_prop('/controller/properties/platformTargetTemperature', 1.7)
"""
import sys
import os
import instrument
import genericcryostat

Ports = instrument.Rest_Ports


@genericcryostat.register
class Fsr(genericcryostat.GenericCryostat):
    def __init__(self, ip, version='v1', verbose=False, tunnel=False, port=Ports.chronos_hlm):
        super().__init__(ip=ip,
                         port=port,
                         version=version,
                         verbose=verbose,
                         tunnel=tunnel)

    def get_cold_head_pressure(self):
        return self.get_prop('/vacuumSystem/vacuumGauges/coldHeadPressure/properties/pressureSample')['pressureSample']['pressure']
    
    def get_sample_chamber_pressure(self):
        return self.get_prop('/vacuumSystem/vacuumGauges/sampleChamberPressure/properties/pressureSample')['pressureSample']['pressure']




from typing import Union, List, Dict
from pymodaq.control_modules.move_utility_classes import (DAQ_Move_base, comon_parameters_fun,
                                                          main, DataActuatorType, DataActuator)

from pymodaq_utils.utils import ThreadCommand
from pymodaq_utils.logger import set_logger, get_module_name

from pymodaq_data import Q_
from pymodaq_gui.parameter import Parameter

from pymodaq_plugins_montana.hardware.scryostation import SCryostation
from pymodaq_plugins_montana.utils import Config as PluginConfig


plugin_config = PluginConfig()
logger = set_logger(get_module_name(__file__))


class DAQ_Move_CryoTemperature(DAQ_Move_base):
    """ Instrument plugin class for an actuator.
    
    This object inherits all functionalities to communicate with PyMoDAQ’s DAQ_Move module through inheritance via
    DAQ_Move_base. It makes a bridge between the DAQ_Move module and the Python wrapper of a particular instrument.


    Attributes:
    -----------
    controller: object
        The particular object that allow the communication with the hardware, in general a python wrapper around the
         hardware library.
             """
    is_multiaxes = True
    _axis_names: Union[List[str], Dict[str, int]] = ['ATSM',]
    _controller_units: Union[str, List[str]] = 'K'
    _epsilon: Union[float, List[float]] = 0.01
    data_actuator_type = DataActuatorType.DataActuator

    params = [
         {'title': 'Server IP:', 'name': 'ipaddress', 'type': 'str', 'value': plugin_config('ip_address')},
         {'title': 'Cryo. Port:', 'name': 'cryo_port', 'type': 'int', 'value': plugin_config('cryo_port')},
         {'title': 'Temp. Stability', 'name': 'stability', 'type': 'float',
          'value': plugin_config('temp_stability'), 'suffix': 'K',
          'siPrefix': True}
             ] + comon_parameters_fun(is_multiaxes, axis_names=_axis_names, epsilon=_epsilon)

    def ini_attributes(self):
        self.controller: SCryostation = None

    def get_actuator_value(self):
        """Get the current value from the hardware with scaling conversion.

        Returns
        -------
        float: The position obtained after scaling conversion.
        """
        pos = DataActuator(data=self.controller.get_user1_temperature()[1])  # when writing your own plugin replace this line
        pos = self.get_position_with_scaling(pos)
        return pos

    def user_condition_to_reach_target(self) -> bool:
        """ Implement a user defined condition for exiting the polling mechanism and specifying
        that the target value has been reached (on top of the existing epsilon mechanism)

        Should be reimplemented in plugins to implement other conditions

        Returns
        -------
        bool: if True, PyMoDAQ considers the target value has been reached
        """
        logger.debug(f'Temp stab is {self.controller.get_user1_temperature_stability()[1]}')
        return self.controller.get_user1_temperature_stability()[1] < self.settings['stability']


    def close(self):
        """Terminate the communication protocol"""
        if self.is_master:
            self.controller.set_user1_temperature_controller_enabled(False)
            self.controller.close()

    def commit_settings(self, param: Parameter):
        """Apply the consequences of a change of value in the detector settings

        Parameters
        ----------
        param: Parameter
            A given parameter (within detector_settings) whose value has been changed by the user
        """
        pass

    def ini_stage(self, controller=None):
        """Actuator communication initialization

        Parameters
        ----------
        controller: (object)
            custom object of a PyMoDAQ plugin (Slave case). None if only one actuator by controller (Master case)

        Returns
        -------
        info: str
        initialized: bool
            False if initialization failed otherwise True
        """
        if self.is_master:  # is needed when controller is master
            self.controller = SCryostation(self.settings['ipaddress'], port=self.settings['cryo_port'])
        else:
            self.controller = controller

        self.controller.set_user1_temperature_controller_enabled(True)
        self.settings.child('timeout').setValue(1000) # allows for temperature stabilisation

        self.emit_status(ThreadCommand('update_ui', attribute='set_abs_value_red',
                                       args=[Q_(plugin_config('target_temp_1'),
                                                self.axis_unit)]))

        self.emit_status(ThreadCommand('update_ui', attribute='set_abs_value_green',
                                       args=[Q_(plugin_config('target_temp_2'),
                                                self.axis_unit)]))

        info = "Cryo connected"
        initialized = True
        return info, initialized

    def move_abs(self, value: DataActuator):
        """ Move the actuator to the absolute target defined by value

        Parameters
        ----------
        value: (float) value of the absolute target positioning
        """

        value = self.check_bound(value)  #if user checked bounds, the defined bounds are applied here
        self.target_value = value
        value = self.set_position_with_scaling(value)  # apply scaling if the user specified one
        self.controller.set_user1_target_temperature(value.value())  # when writing your own plugin replace this line

    def move_rel(self, value: DataActuator):
        """ Move the actuator to the relative target actuator value defined by value

        Parameters
        ----------
        value: (float) value of the relative target positioning
        """
        value = self.check_bound(self.current_position + value) - self.current_position
        self.target_value = value + self.current_position
        value = self.set_position_relative_with_scaling(value)

        self.controller.set_user1_target_temperature(self.target_value.value())

    def move_home(self):
        """Call the reference method of the controller"""

        self.controller.set_user1_target_temperature(290.)

    def stop_motion(self):
      """Stop the actuator and emits move_done signal"""
      self.controller.set_user1_target_temperature(self.controller.get_user1_temperature()[1])


if __name__ == '__main__':
    main(__file__)



from pathlib import Path
from .utils import Config
from pymodaq_utils.utils import get_version, PackageNotFoundError
from pymodaq_utils.logger import set_logger, get_module_name

config = Config()
try:
    __version__ = get_version(__package__)
except PackageNotFoundError:
    __version__ = '0.0.0dev'






import numpy as np

from pymodaq_utils.utils import ThreadCommand
from pymodaq_data.data import DataToExport
from pymodaq_data import Q_
from pymodaq_gui.parameter import Parameter

from pymodaq.control_modules.viewer_utility_classes import DAQ_Viewer_base, comon_parameters, main
from pymodaq.utils.data import DataFromPlugins


from pymodaq_plugins_montana.hardware.scryostation import SCryostation
from pymodaq_plugins_montana.utils import Config as PluginConfig


plugin_config = PluginConfig()


class DAQ_0DViewer_CryoInfo(DAQ_Viewer_base):
    """ Instrument plugin class for a OD viewer.
    
    This object inherits all functionalities to communicate with PyMoDAQ’s DAQ_Viewer module through inheritance via
    DAQ_Viewer_base. It makes a bridge between the DAQ_Viewer module and the Python wrapper of a particular instrument.

    Attributes:
    -----------
    controller: SCryostation
        The particular object that allow the communication with the hardware, in general a python wrapper around the
         hardware library.

    """
    params = comon_parameters+[
        {'title': 'Server IP:', 'name': 'ipaddress', 'type': 'str', 'value': plugin_config('ip_address')},
        {'title': 'Cryo. Port:', 'name': 'cryo_port', 'type': 'int', 'value': plugin_config('cryo_port')},
        {'title': 'Temperatures:', 'name': 'temperatures', 'type': 'group', 'children':[
            {'title': 'Stage 1:', 'name': 'stage1', 'type': 'float', 'value': 290., 'suffix': 'K'},
            {'title': 'Stage 2:', 'name': 'stage2', 'type': 'float', 'value': 290., 'suffix': 'K'},
            {'title': 'Platform:', 'name': 'platform', 'type': 'float', 'value': 290., 'suffix': 'K'},
            {'title': 'User 1:', 'name': 'user1', 'type': 'float', 'value': 290., 'suffix': 'K'},
        ]},
        {'title': 'Pressure:', 'name': 'pressure', 'type': 'float', 'value': 1000., 'suffix': 'mbar'},
        ]

    def ini_attributes(self):
        self.controller: SCryostation = None

        self.temperatures = DataFromPlugins('Temperatures', data=[np.array([290]),
                                                                  np.array([290]),
                                                                  np.array([290]),
                                                                  np.array([290])],
                                            units='K',
                                            labels=['Stage 1', 'Stage 2', 'Platform', 'User 1'])

        self.pressure = DataFromPlugins('Pressure', data=[np.array([1000, ])],
                                        units='mbar',
                                        labels=['pressure'])

    def commit_settings(self, param: Parameter):
        """Apply the consequences of a change of value in the detector settings

        Parameters
        ----------
        param: Parameter
            A given parameter (within detector_settings) whose value has been changed by the user
        """
        pass

    def update_temperatures(self):

        temps = [self.controller.get_stage1_temperature()[1],
                 self.controller.get_stage2_temperature()[1],
                 self.controller.get_platform_temperature()[1],
                 self.controller.get_user1_temperature()[1]]

        self.temperatures.data = [np.array([temp]) for temp in temps]

        self.settings.child('temperatures', 'stage1').setValue(temps[0])
        self.settings.child('temperatures', 'stage2').setValue(temps[1])
        self.settings.child('temperatures', 'platform').setValue(temps[2])
        self.settings.child('temperatures', 'user1').setValue(temps[3])

    def update_pressure(self):
        pressure = Q_(self.controller.get_sample_chamber_pressure(), 'Pa').to('mbar').magnitude
        self.settings.child('pressure').setValue(pressure)
        self.pressure.data[0] = np.array([pressure])


    def ini_detector(self, controller=None):
        """Detector communication initialization

        Parameters
        ----------
        controller: (object)
            custom object of a PyMoDAQ plugin (Slave case). None if only one actuator/detector by controller
            (Master case)

        Returns
        -------
        info: str
        initialized: bool
            False if initialization failed otherwise True
        """

        if self.is_master:
            self.controller = SCryostation(self.settings['ipaddress'], port=self.settings['cryo_port'])  #instantiate you driver with whatever arguments are needed
        else:
            self.controller = controller

        self.update_temperatures()
        self.update_pressure()
        self.dte_signal_temp.emit(DataToExport('Cryo', data=[self.temperatures, self.pressure]))

        info = "Cryo connected"
        initialized = True
        return info, initialized

    def close(self):
        """Terminate the communication protocol"""
        if self.is_master:
            self.controller.close()

    def grab_data(self, Naverage=1, **kwargs):
        """Start a grab from the detector

        Parameters
        ----------
        Naverage: int
            Number of hardware averaging (if hardware averaging is possible, self.hardware_averaging should be set to
            True in class preamble and you should code this implementation)
        kwargs: dict
            others optionals arguments
        """
        self.update_temperatures()
        self.update_pressure()
        self.dte_signal.emit(DataToExport('Cryo', data=[self.temperatures, self.pressure]))

    def stop(self):
        """Stop the current grab hardware wise if necessary"""
        return ''


if __name__ == '__main__':
    main(__file__)
